{ #(@)$Id: UnitTestGUITesting.pas,v 1.5 2000/12/05 02:15:21 juanco Exp $ }
{: DUnit: An XTreme testing framework for Delphi programs.
   @author  The DUnit Group.
   @version $Revision: 1.5 $
}
(*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is DUnit.
 *
 * The Initial Developers of the Original Code are Serge Beaumont
 * and Juancarlo Aez.
 * Portions created The Initial Developers are Copyright (C) 1999-2000.
 * Portions created by The DUnit Group are Copyright (C) 2000.
 * All rights reserved.
 *
 * Contributor(s):
 * Serge Beaumont <beaumose@iquip.nl>
 * Juanco Aez <juanco@users.sourceforge.net>
 * The DUnit group at SourceForge <http://dunit.sourceforge.net>
 *
 *)
unit UnitTestGUITesting;

interface
uses
  TestFramework,
  GUITesting,
  GUITestRunner,

  SysUtils,
  Classes,
  Windows,
  Graphics;

const
  rcs_id: string = '#(@)$Id: UnitTestGUITesting.pas,v 1.5 2000/12/05 02:15:21 juanco Exp $';

type
  TDunitDialogCracker = class(TGUITestRunner);

  TGUITestRunnerTests = class(TGUITestCase)
    FRunner :TGUITestRunner;
  protected
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure TestTabOrder;
    procedure RunEmptySuite;
    procedure RunFailureSuite;
  end;

implementation

type
  TFailuresTestCase = class(TTestCase)
  published
    procedure OneSuccess;
    procedure OneFailure;
    procedure SecondFailure;
    procedure OneError;
  end;

{ TGUITestRunnerTests }

procedure TGUITestRunnerTests.SetUp;
begin
  inherited;
  FRunner := TGUITestRunner.Create(nil);
  FRunner.Color   := clWhite;
  FRunner.Caption := 'This Form is being tested';
  FRunner.Left    := FRunner.Left + 200;
  FRunner.Top     := FRunner.Top + 100;
  FRunner.Width   := 200;
  FRunner.Height  := 132;
  FRunner.AutoSaveAction.Checked := False;
  GUI := FRunner;
end;

procedure TGUITestRunnerTests.TearDown;
begin
  GUI := nil;
  FRunner.Free;
  inherited;
end;

procedure TGUITestRunnerTests.TestTabOrder;
begin
  // need to set a test suite, or buttons will be disabled
  FRunner.Suite := TFailuresTestCase.Suite;
  Show;
  CheckFocused(FRunner.RunButton);
  Tab;
  CheckFocused(FRunner.CloseButton);
  Tab;
  CheckFocused(FRunner.SelectAllButton);
  Tab;
  CheckFocused(FRunner.DeselectAllButton);
  Tab;
  CheckFocused(FRunner.SelectFailedButton);
  Tab;
  CheckFocused(FRunner.TestTree);
  Tab;
  CheckFocused(FRunner.ResultsView);
  Tab;
  CheckFocused(FRunner.FailureListView);
  Tab;
  CheckFocused(FRunner.ErrorMessageRTF);
  Tab;
  CheckFocused(FRunner.RunButton);
  Tab;
  CheckTabTo('RunButton');
end;

procedure TGUITestRunnerTests.RunEmptySuite;
begin
  // no suite
  Show;

  CheckEquals( 0,  FRunner.ProgressBar.Position,               'progress bar');
  CheckEquals('',  FRunner.ResultsView.Items[0].SubItems[0],   'tests');
  CheckEquals('',  FRunner.ResultsView.Items[0].SubItems[1],   'run count');
  CheckEquals('',  FRunner.ResultsView.Items[0].SubItems[2],   'failure count');
  CheckEquals('',  FRunner.ResultsView.Items[0].SubItems[3],   'error count');
  CheckEquals( 0,  FRunner.FailureListView.Items.Count,        'failure list');

  Click(FRunner.RunButton);
  // nothing happens
  CheckEquals( 0,  FRunner.ProgressBar.Position,               'progress bar');
  CheckEquals('',  FRunner.ResultsView.Items[0].SubItems[0],   'tests');
  CheckEquals('',  FRunner.ResultsView.Items[0].SubItems[1],   'run count');
  CheckEquals('',  FRunner.ResultsView.Items[0].SubItems[2],   'failure count');
  CheckEquals('',  FRunner.ResultsView.Items[0].SubItems[3],   'error count');
  CheckEquals( 0,  FRunner.FailureListView.Items.Count,        'failure list');

  Click('CloseButton');

  Check(not FRunner.Visible, 'form closed?');
end;

procedure TGUITestRunnerTests.RunFailureSuite;
begin
  FRunner.Suite := TFailuresTestCase.Suite;
  Show;

  CheckEquals( 0,  FRunner.ProgressBar.Position,               'progress bar');
  CheckEquals('4', FRunner.ResultsView.Items[0].SubItems[0],   'tests');
  CheckEquals('',  FRunner.ResultsView.Items[0].SubItems[1],   'run count');
  CheckEquals('',  FRunner.ResultsView.Items[0].SubItems[2],   'failure count');
  CheckEquals('',  FRunner.ResultsView.Items[0].SubItems[3],   'error count');
  CheckEquals( 0,  FRunner.FailureListView.Items.Count,        'failure list');

  EnterKey('R', [ssAlt]);

  CheckEquals( 4,  FRunner.ProgressBar.Position,               'progress bar');
  CheckEquals('4', FRunner.ResultsView.Items[0].SubItems[0],   'tests');
  CheckEquals('4', FRunner.ResultsView.Items[0].SubItems[1],   'run count');
  CheckEquals('2', FRunner.ResultsView.Items[0].SubItems[2],   'failure count');
  CheckEquals('1', FRunner.ResultsView.Items[0].SubItems[3],   'error count');
  CheckEquals( 3,  FRunner.FailureListView.Items.Count,        'failure list');

  EnterKey('C', [ssAlt]);

  Check(not FRunner.Visible, 'form closed?');
end;

{ TFailuresTestCase }

procedure TFailuresTestCase.OneSuccess;
begin
  // do nothing
end;

procedure TFailuresTestCase.OneError;
begin
  raise EAbort.Create('One Error');
end;

procedure TFailuresTestCase.OneFailure;
begin
  fail('One failure');
end;

procedure TFailuresTestCase.SecondFailure;
begin
  fail('Second failure');
end;

initialization
  RegisterTestSuites('GUI Tests', [TGUITestRunnerTests]);
end.
